import asyncio
import json
import re
import time
from pathlib import Path
from typing import Any, Dict, List, Optional, Union

import httpx
import qrcode
from bs4 import BeautifulSoup
from loguru import logger

import WeiboBot.const as const
from WeiboBot.exception import (
    CommentError,
    DeleteCommentError,
    DeleteWeiboError,
    LikeWeiboError,
    PostWeiboError,
    SendMessageError,
    UploadPicError,
    WeiboNotExist,
)
from WeiboBot.model import Chat, ChatDetail, Comment, Page, User, Weibo
from WeiboBot.typing import CID, MID
from WeiboBot.util import get_cookies_value, load_cookies, save_cookies


class NetTool:
    def __init__(
        self, cookies: Union[str, dict, Path] = Path("weibobot_cookies.json")
    ) -> None:
        """初始化网络工具类。

        Args:
            cookies (str): 微博的cookies字符串
        """
        super(NetTool, self).__init__()
        self.client = httpx.AsyncClient(max_redirects=10)
        self.mid: int = 0
        self._last_refresh_token_time = 0
        self._token_refresh_interval = 60 * 10  # 10分钟
        self.cookies_path = cookies
        if isinstance(cookies, str):
            logger.info("从字符串加载cookies")
            cookies_dict = json.loads(cookies)
            for name, value in cookies_dict.items():
                self.client.cookies.set(name, value)
        elif isinstance(cookies, dict):
            logger.info("从字典加载cookies")
            for name, value in cookies.items():
                self.client.cookies.set(name, value)
        elif isinstance(cookies, Path):
            logger.info("从文件加载cookies")
            if not load_cookies(cookies, self.client):
                logger.info("文件不存在，将使用扫码登录")

    async def __aenter__(self):
        await self.login()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.client.aclose()

    async def _refresh_token(self):
        response = await self.client.get(
            "https://m.weibo.cn/api/config", headers={"Referer": "https://m.weibo.cn/"}
        )
        response.raise_for_status()
        for cookie in response.cookies.jar:
            logger.debug(
                f"Cookie: {cookie.name}={cookie.value} "
                f"Domain={cookie.domain} "
                f"Path={cookie.path} "
                f"Expires={cookie.expires}"
            )
            # 直接设置到 client 的 cookies 中
            self.client.cookies.set(
                cookie.name, cookie.value, domain=cookie.domain, path=cookie.path
            )

        # 原有的 token 处理
        result = response.json()
        if result["data"]["login"]:
            token = result["data"]["st"]
            self.client.cookies.set("XSRF-TOKEN", token, domain="m.weibo.cn")
            save_cookies(self.cookies_path, self.client)

    async def get_token(self) -> str:
        now = time.time()
        if now - self._last_refresh_token_time > self._token_refresh_interval:
            await self._refresh_token()
            self._last_refresh_token_time = now
        return get_cookies_value(self.client, "XSRF-TOKEN")

    async def login(self) -> int:
        """登录微博。"""
        if not self.client.cookies:
            await self.login_by_qr_code()
            return await self.check_login_status()
        else:
            return await self.check_login_status()

    async def get_qrcode_url(self) -> str:
        params = {
            "entry": "wapsso",
            "size": "180",
        }
        headers = {
            "Referer": "https://m.weibo.cn/",
        }
        response = await self.client.get(
            "https://passport.weibo.com/sso/v2/qrcode/image",
            params=params,
            headers=headers,
        )
        response.raise_for_status()
        result = response.json()
        qrid = result["data"]["qrid"]
        qrcode_url = f"https://passport.weibo.cn/signin/qrcode/scan?qr={qrid}"
        qrcode_file = "qrcode.png"
        logger.info(f"二维码URL: {qrcode_url}")
        logger.info(
            f"请使用微博扫描二维码登录，如果显示不全，可以打开根目录下的{qrcode_file}文件"
        )
        qr = qrcode.QRCode()
        qr.add_data(qrcode_url)
        qr.make(fit=True)
        qr.print_ascii()
        qr.make_image(fill_color="black", back_color="white").save(qrcode_file)
        return qrid

    async def login_by_qr_code(self):
        qrid = await self.get_qrcode_url()
        while True:
            await asyncio.sleep(1)
            params = {
                "entry": "wapsso",
                "source": "wapsso",
                "url": "https://m.weibo.cn/",
                "qrid": qrid,
            }
            headers = {
                "Referer": "https://m.weibo.cn/",
            }
            response = await self.client.get(
                "https://passport.weibo.com/sso/v2/qrcode/check",
                params=params,
                headers=headers,
            )
            response.raise_for_status()
            result = response.json()
            retcode = result["retcode"]
            if retcode == 50114001:
                logger.debug("二维码未使用")
                continue
            elif retcode == 50114003:
                logger.info("二维码已过期")
                qrid = await self.get_qrcode_url()
            elif retcode == 50114002:
                logger.info("成功扫描，请在手机点击确认以登录")
            else:
                logger.info("二维码已使用")
                logger.info(result)
                alt = result.get("data", {}).get("url")
                logger.info(f"跳转 URL: {alt}")
                # 访问跳转链接，完成登录
                headers = {
                    "Referer": "https://m.weibo.cn/",
                }
                final_response = await self.client.get(
                    alt, follow_redirects=True, headers=headers
                )
                final_response.raise_for_status()
                save_cookies(self.cookies_path, self.client)
                # 此时，client.cookies 中包含了登录后的 Cookies
                return

    async def check_login_status(self) -> int:
        logger.info("检查登录状态……")
        url = "https://m.weibo.cn/api/config"
        headers = {
            "Referer": "https://m.weibo.cn/",
        }
        response = await self.client.get(url, headers=headers)
        response.raise_for_status()
        data = response.json()
        isLogin = data["data"]["login"]
        if not isLogin:
            logger.warning("未登录")
            return 0
        token = data["data"]["st"]
        self.client.cookies.set("XSRF-TOKEN", token)
        self._last_refresh_token_time = time.time()
        self.mid = int(data["data"]["uid"])
        return self.mid

    async def user_info(self, user_id: MID) -> User:
        """获取用户信息。

        Args:
            user_id (int): 用户ID

        Returns:
            User: 用户信息
        """
        headers = {
            "Referer": "https://m.weibo.cn/",
            "x-xsrf-token": await self.get_token(),
        }
        params = {
            "uid": int(user_id),
        }
        response = await self.client.get(
            "https://m.weibo.cn/profile/info", params=params, headers=headers
        )
        response.raise_for_status()
        result = response.json()
        user = User.model_validate(result["data"]["user"])
        user.statuses = [
            Weibo.model_validate(weibo) for weibo in result["data"]["statuses"]
        ]
        return user

    async def post_weibo(self, content: str, visible: const.VISIBLE) -> Optional[Weibo]:
        """发布微博。

        Args:
            content (str): 微博内容
            visible (const.VISIBLE): 可见性设置

        Returns:
            Dict[str, Any]: 发布结果
        """
        params = {
            "content": content,
            "visible": visible.value,
            "st": await self.get_token(),
        }
        headers = {
            "Referer": "https://m.weibo.cn/",
            "x-xsrf-token": await self.get_token(),
        }
        response = await self.client.post(
            "https://m.weibo.cn/api/statuses/update", params=params, headers=headers
        )
        response.raise_for_status()
        result = response.json()

        if result["ok"] == 1:
            weibo = Weibo.model_validate(result["data"])
            return weibo
        else:
            raise PostWeiboError(result["msg"])

    async def repost_weibo(
        self, mid: MID, content: str, dualPost: bool
    ) -> Dict[str, Any]:
        """转发微博。

        Args:
            mid (Union[str, int]): 微博ID
            content (str): 转发内容
            dualPost (bool): 是否同时评论

        Returns:
            Dict[str, Any]: 转发结果
        """
        params = {
            "id": mid,
            "content": content,
            "mid": mid,
            "st": await self.get_token(),
            "dualPost": int(dualPost),
        }

        response = await self.client.post(
            "https://m.weibo.cn/api/statuses/repost", params=params
        )
        response.raise_for_status()
        return response.json()

    async def weibo_info(self, mid: MID, comments_count: int = 0) -> Weibo:
        """获取微博详细信息。

        Args:
            mid (Union[str, int]): 微博ID

        Returns:
            Weibo: 微博信息
        """
        url = f"https://m.weibo.cn/detail/{mid}"
        headers = {
            "Referer": "https://m.weibo.cn/",
            "x-xsrf-token": await self.get_token(),
        }
        response = await self.client.get(url, headers=headers)
        response.raise_for_status()
        r = response.text

        soup = BeautifulSoup(r, "html.parser")
        error_msg = soup.select_one("body > div > p")
        if error_msg and error_msg.get_text().strip() == "微博不存在或暂无查看权限!":
            raise WeiboNotExist(f"微博不存在或暂无查看权限! {mid}")

        result = json.loads(
            re.findall(r"(?<=render_data = \[)[\s\S]*(?=\]\[0\])", r)[0]
        )["status"]
        weibo = Weibo.model_validate(result)
        weibo.comments = await self.get_weibo_comments(mid, comments_count)
        return weibo

    async def get_weibo_comments(self, mid: MID, count: int = 20) -> List[Comment]:
        """获取微博评论。
        Args:
            mid (Union[str, int]): 微博ID
            count (int): 获取评论数量，默认20条，-1表示获取所有评论
        Returns:
            List[Comment]: 评论列表
        """
        comments = []
        max_id = 0
        if count == 0:
            return []
        while True:
            url = "https://m.weibo.cn/comments/hotflow"
            headers = {
                "Referer": "https://m.weibo.cn/",
                "x-xsrf-token": await self.get_token(),
            }
            params = {
                "id": mid,
                "mid": mid,
                "max_id": max_id,
                "max_id_type": 0,
            }

            response = await self.client.get(url, headers=headers, params=params)
            response.raise_for_status()
            result = response.json()

            if not comments:  # 第一次请求
                total = result["data"]["total_number"]
                if count != -1:
                    count = min(count, total)

            new_comments = [
                Comment.model_validate(data)
                for data in result["data"]["data"]
                if data["id"] not in [c.id for c in comments]
            ]
            comments.extend(new_comments)

            max_id = result["data"]["max_id"]
            if max_id == 0 or (count != -1 and len(comments) >= count):
                break

        return comments if count == -1 else comments[:count]

    async def upload_chat_file(self, tuid: MID, file: Path) -> str:
        """上传聊天文件。

        Args:
            tuid (int): 接收者用户ID
            file (Path): 文件路径

        Returns:
            str: 文件ID
        """
        files = {"file": (file.name, open(file, "rb"), "image/jpeg")}
        data = {"tuid": tuid, "st": await self.get_token()}
        headers = {
            "Referer": "https://m.weibo.cn/",
            "x-xsrf-token": await self.get_token(),
        }
        response = await self.client.post(
            "https://m.weibo.cn/api/chat/upload",
            headers=headers,
            data=data,
            files=files,
        )
        response.raise_for_status()
        result = response.json()
        if "fids" not in result.get("data", {}):
            raise UploadPicError()
        return result["data"]["fids"]

    async def upload_comment_pic(self, file: Path) -> str:
        """上传评论图片。

        Args:
            file (Path): 图片文件路径

        Returns:
            str: 图片ID
        """
        files = {"pic": (file.name, open(file, "rb"), "image/jpeg")}
        data = {"type": "json", "st": await self.get_token()}
        headers = {
            "Referer": "https://m.weibo.cn/",
            "x-xsrf-token": await self.get_token(),
        }
        response = await self.client.post(
            "https://m.weibo.cn/api/statuses/uploadPic",
            headers=headers,
            data=data,
            files=files,
        )
        response.raise_for_status()
        result = response.json()
        if "pic_id" not in result:
            raise UploadPicError()
        return result["pic_id"]

    async def send_message(
        self, uid: MID, content: str, file: Optional[Path] = None
    ) -> ChatDetail:
        """发送私信。

        Args:
            uid (Union[str, int]): 接收者用户ID
            content (str): 消息内容
            file (Path): 附件文件路径

        Returns:
            ChatDetail: 发送结果
        """
        params = {
            "uid": int(uid),
            "content": content,
            "st": await self.get_token(),
        }
        headers = {
            "Referer": "https://m.weibo.cn/",
            "x-xsrf-token": await self.get_token(),
        }
        if file and file.exists():
            media_type = const.MEDIA.PHOTO.value
            fids = await self.upload_chat_file(tuid=uid, file=file)
            params["media_type"] = media_type
            params["content"] = ""
            params["fids"] = fids

        response = await self.client.post(
            "https://m.weibo.cn/api/chat/send", params=params, headers=headers
        )
        response.raise_for_status()
        result = response.json()
        if result["ok"] == 1:
            chat_detail = ChatDetail.model_validate(result["data"])
            return chat_detail
        else:
            raise SendMessageError(result["msg"])

    async def user_chat(
        self, uid: MID, since_id: int = 0, is_continuous=0
    ) -> ChatDetail:
        """获取与指定用户的聊天记录。

        Args:
            uid (Union[str, int]): 用户ID
            since_id (int): 起始消息ID

        Returns:
            ChatDetail: 聊天记录
        """
        params = {
            "count": 20,
            "uid": int(uid),
            "since_id": since_id,
            "is_continuous": is_continuous,
        }
        headers = {
            "Referer": "https://m.weibo.cn/",
            "x-xsrf-token": await self.get_token(),
        }
        response = await self.client.get(
            "https://m.weibo.cn/api/chat/list", params=params, headers=headers
        )
        response.raise_for_status()

        data = response.json()
        return ChatDetail.model_validate(data["data"])

    async def chat_list(self, page: int = 1) -> List[Chat]:
        """获取聊天列表。

        Args:
            page (int): 页码

        Returns:
            List[Chat]: 聊天列表
        """
        params = {"page": page}
        headers = {
            "Referer": "https://m.weibo.cn/",
            "x-xsrf-token": await self.get_token(),
        }
        response = await self.client.get(
            "https://m.weibo.cn/message/msglist", params=params, headers=headers
        )
        response.raise_for_status()
        result = response.json()
        chat_list = [Chat.model_validate(chat) for chat in result["data"]]
        return chat_list

    async def mentions_cmt(self, page: int = 1) -> List[Comment]:
        """获取@我的评论。

        Args:
            page (int): 页码

        Returns:
            List[Comment]: @我的评论列表
        """
        params = {"page": page}
        headers = {
            "Referer": "https://m.weibo.cn/",
            "x-xsrf-token": await self.get_token(),
        }
        response = await self.client.get(
            "https://m.weibo.cn/message/mentionsCmt", params=params, headers=headers
        )
        response.raise_for_status()
        data = response.json()
        cmt_list = [Comment.model_validate(cmt) for cmt in data["data"]]
        return cmt_list

    async def refresh_page(self, max_id: int = 0) -> Page:
        """刷新关注页面。

        Args:
            max_id (Union[str, int]): 最大ID

        Returns:
            Page: 关注页面，注意里面的statuses不是完整微博，需要用weibo_info获取
        """
        params = {"max_id": max_id}
        headers = {
            "Referer": "https://m.weibo.cn/",
            "x-xsrf-token": await self.get_token(),
        }
        response = await self.client.get(
            "https://m.weibo.cn/feed/friends", params=params, headers=headers
        )
        response.raise_for_status()
        result = response.json()
        page = Page.model_validate(result["data"])
        return page

    async def like_weibo(self, mid: MID) -> bool:
        """点赞微博。

        Args:
            mid (Union[str, int]): 微博ID

        Returns:
            bool: 点赞结果
        """
        params = {
            "id": mid,
            "attitude": "heart",
            "st": await self.get_token(),
        }
        headers = {
            "Referer": "https://m.weibo.cn/",
            "x-xsrf-token": await self.get_token(),
        }
        response = await self.client.post(
            "https://m.weibo.cn/api/attitudes/create", params=params, headers=headers
        )
        response.raise_for_status()
        result = response.json()
        if result["ok"] == 1:
            return True
        else:
            raise LikeWeiboError(result["msg"])

    async def del_weibo(self, mid: MID) -> bool:
        """删除微博。

        Args:
            mid (Union[str, int]): 微博ID

        Returns:
            Dict[str, Any]: 删除结果
        """
        params = {"mid": mid, "st": await self.get_token()}
        headers = {
            "Referer": "https://m.weibo.cn/",
            "x-xsrf-token": await self.get_token(),
        }
        response = await self.client.post(
            "https://m.weibo.cn/profile/delMyblog", params=params, headers=headers
        )
        response.raise_for_status()
        result = response.json()
        if result["ok"] == 1:
            return True
        else:
            raise DeleteWeiboError(result["msg"])

    async def comment_weibo(
        self, mid: MID, content: str, file: Optional[Path] = None
    ) -> Comment:
        """评论微博。

        Args:
            mid (Union[str, int]): 微博ID
            content (str): 评论内容
            file_path (str, optional): 图片文件路径. 默认为空字符串.

        Returns:
            Comment: 评论结果
        """
        params = {
            "id": mid,
            "mid": mid,
            "content": content,
            "st": await self.get_token(),
        }

        if file and file.exists():
            pic_id = await self.upload_comment_pic(file=file)
            params["picId"] = pic_id
        headers = {
            "Referer": "https://m.weibo.cn/",
            "x-xsrf-token": await self.get_token(),
        }
        response = await self.client.post(
            "https://m.weibo.cn/api/comments/create", params=params, headers=headers
        )
        response.raise_for_status()
        result = response.json()
        if result["ok"] == 1:
            comment = Comment.model_validate(result["data"])
            return comment
        else:
            raise CommentError(result["msg"])

    async def del_comment(self, cid: CID) -> bool:
        """删除评论。

        Args:
            cid (Union[str, int]): 评论ID

        Returns:
            Dict[str, Any]: 删除结果
        """
        params = {"cid": int(cid), "st": await self.get_token()}
        headers = {
            "Referer": "https://m.weibo.cn/",
            "x-xsrf-token": await self.get_token(),
        }
        response = await self.client.post(
            "https://m.weibo.cn/comments/destroy", params=params, headers=headers
        )
        response.raise_for_status()
        result = response.json()
        if result["ok"] == 1:
            return True
        else:
            raise DeleteCommentError(result["msg"])



from typing import TYPE_CHECKING, List, Optional, Union

from pydantic import BaseModel, Field

if TYPE_CHECKING:
    from WeiboBot.model.user import User

class Comment(BaseModel):
    """评论模型"""

    id: int = Field(description="评论ID")
    mid: str = Field(description="评论MID")
    created_at: str = Field(description="评论创建时间")
    text: str = Field(description="评论内容")
    source: str = Field(description="评论来源")
    reply_id: Optional[int] = Field(None, description="回复的评论ID")
    reply_text: Optional[str] = Field(None, description="回复的评论内容")
    pics: Optional[List[str]] = Field(None, description="评论图片列表")
    like_counts: int = Field(0, description="点赞数")
    is_liked: bool = Field(False, description="是否已点赞")
    is_followed: bool = Field(False, description="是否已关注评论用户")
    user: "User" = Field(None, description="评论用户")
    comments: Union[List["Comment"], None,bool] = Field(None, description="子评论列表")
    class Config:
        extra = "ignore"



from enum import Enum


class VISIBLE(Enum):
    ALL = 0  # 全部
    ONLY_ME = 1  # 仅自己可见
    FRIENDS = 6  # 仅好友可见
    FOLLOWERS = 10  # 仅关注人可见


class MSG(Enum):
    NORMAL = 1  # 普通消息
    SUBSCRIPTION = 4  # 订阅消息


class ACTION(Enum):
    UNDONE = 0  # 未完成
    RUNNING = 1  # 运行中
    DONE = 2  # 已完成
    FAILED = 3  # 失败
    MAX_TRY = 4  # 超过最大尝试次数


class WEIBO_ERR(Enum):
    DEFAULT = 0  # 默认
    NO_DATA = 100011  # 没有数据
    NO_CONTENT = 100010  # 没有数据
    NO_EXIST = 20101  # 微博不存在或暂无查看权限


class MEDIA(Enum):
    NONE = 0
    PHOTO = 1



from .comment import Comment
from .message import Chat, ChatDetail, Message
from .user import User
from .weibo import Page, Weibo

__all__ = ["User", "Weibo", "Comment", "Message", "Chat", "Page", "ChatDetail"]



import json
from pathlib import Path

import httpx


def save_cookies(cookies_path: Path, client: httpx.AsyncClient):
    """保存cookies到文件"""
    cookies_dict = {}
    for cookie in client.cookies.jar:
        cookies_dict[cookie.name] = cookie.value
    cookies_path.parent.mkdir(parents=True, exist_ok=True)
    cookies_path.touch(exist_ok=True)
    # 根据cookies_dict的key排序
    cookies_dict = dict(sorted(cookies_dict.items()))
    with open(cookies_path, "w", encoding="utf-8") as f:
        json.dump(cookies_dict, f, ensure_ascii=False, indent=4)


def load_cookies(cookies_path: Path, client: httpx.AsyncClient) -> bool:
    if not cookies_path.exists():
        return False
    """从文件加载cookies"""
    with open(cookies_path, "r", encoding="utf-8") as f:
        cookies_dict = json.load(f)
    for name, value in cookies_dict.items():
        client.cookies.set(name, value)
    return True


def get_cookies_value(client: httpx.AsyncClient, name: str) -> str:
    for cookie in client.cookies.jar:
        if cookie.name == name:
            return cookie.value
    return ""



class WeiboNotExist(Exception):
    pass


class PostWeiboError(Exception):
    pass


class CommentError(Exception):
    pass


class DeleteCommentError(Exception):
    pass


class UploadPicError(Exception):
    pass


class DeleteWeiboError(Exception):
    pass


class SendMessageError(Exception):
    pass


class LikeWeiboError(Exception):
    pass



from typing import TYPE_CHECKING, List, Optional

from pydantic import BaseModel, Field

if TYPE_CHECKING:
    from WeiboBot.model.weibo import Weibo


class User(BaseModel):
    id: int = Field(description="用户id")
    screen_name: str = Field(description="用户昵称")
    profile_image_url: str = Field(description="用户头像")
    profile_url: str = Field(description="用户主页")
    statuses_count: int = Field(description="微博数")
    verified: bool = Field(description="是否是微博认证用户")
    verified_type: int = Field(description="认证类型")
    close_blue_v: bool = Field(description="是否关注微博蓝V")
    description: str = Field(description="用户描述")
    gender: str = Field(description="性别")
    mbtype: int = Field(description="会员类型")
    urank: int = Field(description="用户等级")
    mbrank: int = Field(description="会员等级")
    follow_me: int = Field(description="是否关注我")
    following: int = Field(description="我是否关注")
    follow_count: int = Field(description="关注数")
    followers_count: str = Field(description="粉丝数")
    followers_count_str: str = Field(description="粉丝数")
    cover_image_phone: str = Field(description="主页头图")
    avatar_hd: str = Field(description="高清头像")
    like: bool = Field(description="是否喜欢")
    like_me: bool = Field(description="是否喜欢我")
    badge: Optional[dict] = Field(default_factory=dict, description="徽章")
    verified_type_ext: Optional[int] = Field(default=None, description="认证类型扩展")
    verified_reason: str = Field(description="认证原因")
    statuses: Optional[List["Weibo"]] = Field(
        default_factory=list, description="最新微博"
    )

    class Config:
        extra = "ignore"

    def __str__(self) -> str:
        result = f"用户名:{self.screen_name},关注:{self.follow_count},粉丝:{self.followers_count},微博数量:{self.statuses_count}\n"
        result += f"微博简介:{self.description}\n"
        result += f"微博地址:{self.profile_url}\n"
        result += f"微博头像:{self.profile_image_url}\n"
        result += f"微博背景图:{self.cover_image_phone}\n"
        return result



from .bot import Bot
from .model import Chat, ChatDetail, Comment, Page, User, Weibo
from .net import NetTool

__all__ = ["Bot", "NetTool", "Comment", "User", "Weibo", "Chat", "ChatDetail", "Page"]

Chat.model_rebuild()
Comment.model_rebuild()
User.model_rebuild()
Weibo.model_rebuild()
Page.model_rebuild()
ChatDetail.model_rebuild()



from tortoise import fields, models


class WeiboRead(models.Model):
    id = fields.IntField(pk=True)
    mid = fields.IntField()

    class Meta:
        table = "weibo_read"

    @classmethod
    async def create_record(cls, mid: int) -> "WeiboRead":
        """创建一条微博阅读记录"""
        return await cls.create(mid=mid)

    @classmethod
    async def get_by_mid(cls, mid: int) -> "WeiboRead":
        """根据微博ID查询记录"""
        return await cls.filter(mid=mid).first()


class MentionCmtRead(models.Model):
    id = fields.IntField(pk=True)
    mid = fields.IntField()

    class Meta:
        table = "mention_cmt_read"

    @classmethod
    async def create_record(cls, mid: int) -> "MentionCmtRead":
        """创建一条@评论阅读记录"""
        return await cls.create(mid=mid)

    @classmethod
    async def get_by_mid(cls, mid: int) -> "MentionCmtRead":
        """根据@评论ID查询记录"""
        return await cls.filter(mid=mid).first()


class WeiboRepost(models.Model):
    id = fields.IntField(pk=True)
    mid = fields.IntField()

    class Meta:
        table = "weibo_repost"

    @classmethod
    async def create_record(cls, mid: int) -> "WeiboRepost":
        """创建一条微博转发记录"""
        return await cls.create(mid=mid)

    @classmethod
    async def get_by_mid(cls, mid: int) -> "WeiboRepost":
        """根据微博ID查询记录"""
        return await cls.filter(mid=mid).first()



from typing import TYPE_CHECKING, List

from pydantic import BaseModel, Field

if TYPE_CHECKING:
    from .weibo import Weibo


class Page(BaseModel):
    max_id: int = Field(description="最大ID")
    interval: int = Field(description="间隔")
    since_id: int = Field(description="起始ID")
    statuses: List["Weibo"] = Field(description="微博列表")



from typing import Awaitable, Callable, List, Tuple, TypeAlias

from WeiboBot.model import Chat, Comment, Weibo

# 定义类型别名
MessageHandler: TypeAlias = Callable[[Chat], Awaitable[None]]
WeiboHandler: TypeAlias = Callable[[Weibo], Awaitable[None]]
MentionCommentHandler: TypeAlias = Callable[[Comment], Awaitable[None]]
TickHandler: TypeAlias = Callable[[], Awaitable[None]]


class EventManager:
    def __init__(self):
        self.msg_handler: List[Tuple[int, Callable]] = []
        self.weibo_handler: List[Tuple[int, Callable]] = []
        self.mention_cmt_handler: List[Tuple[int, Callable]] = []
        self.tick_handler: List[Tuple[int, Callable]] = []

    def onNewMsg(self, priority: int = 10):
        """注册新消息处理函数

        Args:
            priority (int, optional): 优先级，数字越大优先级越高. 默认为10.

        Returns:
            Callable: 装饰器函数
        """

        def decorator(func: Callable):
            self.msg_handler.append((priority, func))
            # 按优先级降序排序
            self.msg_handler.sort(key=lambda x: x[0], reverse=True)
            return func

        return decorator

    def onNewWeibo(self, priority: int = 10):
        """注册新微博处理函数

        Args:
            priority (int, optional): 优先级，数字越大优先级越高. 默认为10.

        Returns:
            Callable: 装饰器函数
        """

        def decorator(func: Callable):
            self.weibo_handler.append((priority, func))
            # 按优先级降序排序
            self.weibo_handler.sort(key=lambda x: x[0], reverse=True)
            return func

        return decorator

    def onMentionCmt(self, priority: int = 10):
        """注册@评论处理函数

        Args:
            priority (int, optional): 优先级，数字越大优先级越高. 默认为10.

        Returns:
            Callable: 装饰器函数
        """

        def decorator(func: Callable):
            self.mention_cmt_handler.append((priority, func))
            # 按优先级降序排序
            self.mention_cmt_handler.sort(key=lambda x: x[0], reverse=True)
            return func

        return decorator

    def onTick(self, priority: int = 10):
        """注册定时任务处理函数

        Args:
            priority (int, optional): 优先级，数字越大优先级越高. 默认为10.

        Returns:
            Callable: 装饰器函数
        """

        def decorator(func: Callable):
            self.tick_handler.append((priority, func))
            # 按优先级降序排序
            self.tick_handler.sort(key=lambda x: x[0], reverse=True)
            return func

        return decorator



from pathlib import Path

from tortoise import Tortoise


async def init_db(db_path: Path):
    await Tortoise.init(
        db_url=f"sqlite://{db_path}",
        modules={"models": ["WeiboBot.data.record"]},
    )
    await Tortoise.generate_schemas()



import asyncio
import time
from functools import wraps
from pathlib import Path
from typing import Any, Callable, List, Optional, Union

from loguru import logger

from WeiboBot.bot.event import EventManager
from WeiboBot.data import MentionCmtRead, WeiboRead, WeiboRepost, init_db
from WeiboBot.model import Chat, User, Weibo
from WeiboBot.net import NetTool


def interval_control(interval: float):
    """控制函数执行间隔的装饰器

    Args:
        interval (float): 执行间隔（秒）
    """
    last_run = {}

    def decorator(func: Callable) -> Callable:
        @wraps(func)
        async def wrapper(*args, **kwargs) -> Any:
            now = time.time()
            func_name = func.__name__

            # 如果是第一次执行或者已经超过间隔时间
            if func_name not in last_run or now - last_run[func_name] >= interval:
                last_run[func_name] = now
                return await func(*args, **kwargs)

        return wrapper

    return decorator


class Bot(NetTool):
    def __init__(
        self,
        cookies: Union[str, dict, Path] = Path("weibobot_cookies.json"),
        db_path: Path = "weibo_bot.db",
    ):
        super(Bot, self).__init__(cookies)
        self.event_manager = EventManager()
        self.weibo_read = set()
        self.db_path: Path = db_path
        self.mid: int = 0
        self.bot_info: Optional[User] = None

    async def __aenter__(self):
        await init_db(self.db_path)
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await super().__aexit__(exc_type, exc_val, exc_tb)

    # region 数据库操作
    async def is_weibo_read(self, mid: Union[str, int]) -> bool:
        weibo_read = await WeiboRead.get_by_mid(mid)
        return weibo_read is not None

    async def mark_weibo(self, mid: Union[str, int]):
        mid = int(mid)
        await WeiboRead.create_record(mid)

    async def is_mention_cmt_read(self, mid: Union[str, int]) -> bool:
        mention_cmt_read = await MentionCmtRead.get_by_mid(mid)
        return mention_cmt_read is not None

    async def mark_mention_cmt(self, mid: Union[str, int]):
        mid = int(mid)
        await MentionCmtRead.create_record(mid)

    async def is_weibo_repost(self, mid: Union[str, int]) -> bool:
        weibo_repost = await WeiboRepost.get_by_mid(mid)
        return weibo_repost is not None

    async def mark_weibo_repost(self, mid: Union[str, int]):
        mid = int(mid)
        await WeiboRepost.create_record(mid)

    # endregion

    # region 事件处理函数
    @interval_control(5)  # 聊天检查间隔30秒
    async def chat_loop(self):
        chat_list: List[Chat] = await self.chat_list()
        for chat in chat_list:
            if chat.unread > 0 and chat.scheme.find("gid") == -1:
                chat_detail = await self.user_chat(chat.user.id)
                if chat_detail is None:
                    logger.warning(f"获取聊天失败:{chat.id}")
                    continue
                chat_detail.msgs = [
                    msg for msg in chat_detail.msgs[: chat.unread] if msg.dm_type == 1
                ]
                for _, func in self.event_manager.msg_handler:
                    await func(chat_detail)

    @interval_control(5)  # @评论检查间隔10分钟
    async def mentions_cmt_loop(self):
        cmt_list = await self.mentions_cmt()
        for cmt in cmt_list:
            if await self.is_mention_cmt_read(cmt.mid):
                continue
            for _, func in self.event_manager.mention_cmt_handler:
                await func(cmt)
            await self.mark_mention_cmt(cmt.mid)

    @interval_control(5)  # 页面扫描间隔5秒
    async def scan_pages_loop(self):
        page = await self.refresh_page()
        if not page or not page.statuses:
            return
        for weibo in page.statuses:
            if await self.is_weibo_read(weibo.id):
                continue
            for _, func in self.event_manager.weibo_handler:
                await func(weibo)
            await self.mark_weibo(weibo.id)

    @interval_control(1)  # 定时任务间隔1秒
    async def tick_loop(self):
        for _, func in self.event_manager.tick_handler:
            await func()

    # endregion

    # region 重载功能

    async def login(self) -> int:
        self.mid = await super().login()
        return self.mid

    async def repost_weibo(
        self, mid: Union[str, int], content: str = "转发微博", dualPost: bool = False
    ) -> Union[Weibo, None]:
        result = await super().repost_weibo(mid, content, dualPost)
        await self.mark_weibo_repost(mid)
        return result

    # endregion

    # region 事件装饰器
    def onNewMsg(self, priority: int = 0):
        return self.event_manager.onNewMsg(priority)

    def onNewWeibo(self, priority: int = 0):
        return self.event_manager.onNewWeibo(priority)

    def onMentionCmt(self, priority: int = 0):
        return self.event_manager.onMentionCmt(priority)

    def onTick(self, priority: int = 0):
        return self.event_manager.onTick(priority)

    # endregion

    # region 生命周期管理
    async def lifecycle(self):
        await init_db(self.db_path)
        mid = await self.login()
        if mid == 0:
            logger.error("登录失败")
            await self.login_by_qr_code()
            await self.check_login_status()
        while True:
            try:
                await asyncio.wait_for(
                    asyncio.gather(
                        self.chat_loop(),
                        self.scan_pages_loop(),
                        self.mentions_cmt_loop(),
                        self.tick_loop(),
                    ),
                    timeout=30,
                )
                logger.debug("Heartbeat - 所有事件处理完成")
            except asyncio.TimeoutError:
                logger.warning("事件处理超时")
            finally:
                await asyncio.sleep(1)

    def run(self):
        loop = asyncio.get_event_loop()
        try:
            loop.run_until_complete(self.lifecycle())
        except KeyboardInterrupt:
            logger.info("收到退出信号，正在关闭...")
            loop.close()
            logger.info("已安全退出")

    # endregion



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Dict, List, Optional

from pydantic import BaseModel, Field

from WeiboBot.typing import MID

if TYPE_CHECKING:
    from WeiboBot.model import User


class ChatUser(BaseModel):
    id: int = Field(description="用户ID/群ID")
    screen_name: str = Field(description="用户昵称/群昵称")
    avatar_large: str = Field(description="头像/群头像")
    remark: str = Field(description="备注")
    verified: bool = Field(description="是否认证")
    verified_type: int = Field(description="认证类型")
    verified_type_ext: int = Field(description="认证类型扩展")


class Chat(BaseModel):
    created_at: str = Field(description="创建时间")
    scheme: str = Field(description="详情url")
    unread: int = Field(description="未读消息数")
    text: str = Field(description="消息内容")
    user: ChatUser = Field(description="用户")

    class Config:
        extra = "ignore"


class Message(BaseModel):
    created_at: str = Field(description="创建时间")
    dm_type: int = Field(description="消息类型")
    id: str = Field(description="消息ID")
    media_type: int = Field(description="媒体类型")
    msg_status: int = Field(description="消息状态")
    recipient_id: int = Field(description="收件人ID")
    recipient_screen_name: str = Field(description="收件人的昵称")
    sender_id: int = Field(description="发送者ID")
    sender_screen_name: str = Field(description="发送者的昵称")
    text: str = Field(description="内容")
    attachment: Dict[str, Any] = Field(default=None, description="附件")

    class Config:
        extra = "ignore"


class ChatDetail(BaseModel):
    total_number: Optional[int] = Field(description="总数")
    following: Optional[bool] = Field(description="是否关注")
    last_read_mid: Optional[int] = Field(description="最后阅读的MID")
    title: str = Field(description="标题")
    users: Dict[MID, "User"] = Field(description="用户列表")
    msgs: List[Message] = Field(description="消息列表")



from typing import TYPE_CHECKING, List, Optional

from pydantic import BaseModel, Field


if TYPE_CHECKING:
    from WeiboBot.model.user import User
    from WeiboBot.model.comment import Comment

class Weibo(BaseModel):
    """微博模型，只保留最常用的字段"""

    visible: dict = Field(description="微博的可见性及指定可见分组信息")
    created_at: str = Field(description="微博创建时间")
    id: str = Field(description="微博ID")
    mid: str = Field(description="微博MID")
    text: str = Field(description="微博信息内容")
    source: Optional[str] = Field(default="", description="微博来源")
    user: Optional["User"] = Field(default=None, description="微博作者的用户信息字段")
    reposts_count: Optional[int] = Field(default=0, description="转发数")
    comments_count: Optional[int] = Field(default=0, description="评论数")
    attitudes_count: Optional[int] = Field(default=0, description="赞数")
    isLongText: Optional[bool] = Field(default=False, description="是否长文本")
    liked: Optional[bool] = Field(default=False, description="是否已赞")
    pics: Optional[list[dict]] = Field(default_factory=list, description="配图")
    page_info: Optional[dict] = Field(default_factory=dict, description="页面信息")
    longText: Optional[dict] = Field(default_factory=dict, description="长文本")
    retweeted_status: Optional[dict] = Field(
        default_factory=dict, description="转发状态"
    )
    deleted: Optional[str] = Field(default="0", description="是否已删除")
    retweeted_status: Optional["Weibo"] = Field(default=None, description="原始微博")
    comments: Optional[List["Comment"]] = Field(default=None, description="评论列表")
    class Config:
        extra = "ignore"

    def detail_url(self) -> str:
        """获取微博详情页URL"""
        return f"https://m.weibo.cn/detail/{self.id}"

    def full_text(self) -> str:
        """获取完整文本内容"""
        if self.longText != {}:
            return self.longText["longTextContent"]
        else:
            return self.text

    def weibo_id(self) -> int:
        """获取微博ID（整数形式）"""
        return int(self.id)

    def user_uid(self) -> int:
        """获取用户ID"""
        return int(self.user["id"])

    def video_url(self) -> str:
        """获取视频URL"""
        url = ""
        if self.page_info.get("type", "") == "video" and "urls" in self.page_info:
            url = list(self.page_info["urls"].values())[0]
        return url

    def image_list(self) -> List[str]:
        """获取图片URL列表"""
        return [img["large"]["url"] for img in self.pics] if self.pics else []

    def thumbnail_image_list(self) -> List[str]:
        """获取缩略图URL列表"""
        return [img["url"] for img in self.pics] if self.pics else []

    def is_visible(self) -> bool:
        """检查微博是否可见"""
        return self.visible.get("type", 0) == 0
